@@ -1,13 +1,11 @@
 /*
    +----------------------------------------------------------------------+
-   | PHP Version 5                                                        |
-   +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2012 The PHP Group                                |
+   | Copyright (c) The PHP Group                                          |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
    | available through the world-wide-web at the following url:           |
-   | http://www.php.net/license/3_01.txt                                  |
+   | https://www.php.net/license/3_01.txt                                 |
    | If you did not receive a copy of the PHP license and are unable to   |
    | obtain it through the world-wide-web, please send a note to          |
    | license@php.net so we can mail you a copy immediately.               |
@@ -16,8 +14,6 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: com_dotnet.c 321634 2012-01-01 13:15:04Z felipe $ */
-
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -53,17 +49,17 @@ typedef struct _Imscorlib_System_AppDomain IAppDomain;
 struct _Imscorlib_System_AppDomainVtbl {
 	BEGIN_INTERFACE
 
-	HRESULT ( STDMETHODCALLTYPE *QueryInterface )( 
+	HRESULT ( STDMETHODCALLTYPE *QueryInterface )(
 		IAppDomain * This,
 		/* [in] */ REFIID riid,
 		/* [iid_is][out] */ void **ppvObject);
-        
-	ULONG ( STDMETHODCALLTYPE *AddRef )( 
+
+	ULONG ( STDMETHODCALLTYPE *AddRef )(
 		IAppDomain * This);
-        
-	ULONG ( STDMETHODCALLTYPE *Release )( 
+
+	ULONG ( STDMETHODCALLTYPE *Release )(
 		IAppDomain * This);
-        
+
 	/* this is padding to get CreateInstance into the correct position */
 #define DUMMY_METHOD(x)		HRESULT ( STDMETHODCALLTYPE *dummy_##x )(IAppDomain *This)
 
@@ -121,7 +117,45 @@ struct dotnet_runtime_stuff {
 	DISPID create_instance;
 };
 
-static HRESULT dotnet_init(char **p_where TSRMLS_DC)
+/* We link dynamically to mscoree.dll to avoid the hard dependency on .NET
+ * framework, which is only required if a dotnet instance is to be created.
+ */
+static HRESULT dotnet_bind_runtime(LPVOID FAR *ppv)
+{
+	HRESULT hr;
+	HMODULE mscoree;
+	typedef HRESULT (STDAPICALLTYPE *cbtr_t)(LPCWSTR pwszVersion, LPCWSTR pwszBuildFlavor, REFCLSID rclsid, REFIID riid, LPVOID FAR *ppv);
+	cbtr_t CorBindToRuntime;
+	OLECHAR *oleversion;
+	char *version;
+
+	mscoree = LoadLibraryA("mscoree.dll");
+	if (mscoree == NULL) {
+		return S_FALSE;
+	}
+
+	CorBindToRuntime = (cbtr_t) GetProcAddress(mscoree, "CorBindToRuntime");
+	if (CorBindToRuntime == NULL) {
+		FreeLibrary(mscoree);
+		return S_FALSE;
+	}
+
+	version = INI_STR("com.dotnet_version");
+	if (version == NULL || *version == '\0') {
+		oleversion = NULL;
+	} else {
+		oleversion = php_com_string_to_olestring(version, strlen(version), COMG(code_page));
+	}
+
+	hr = CorBindToRuntime(oleversion, NULL, &CLSID_CorRuntimeHost, &IID_ICorRuntimeHost, ppv);
+
+	efree(oleversion);
+	FreeLibrary(mscoree);
+
+	return hr;
+}
+
+static HRESULT dotnet_init(char **p_where)
 {
 	HRESULT hr;
 	struct dotnet_runtime_stuff *stuff;
@@ -134,10 +168,8 @@ static HRESULT dotnet_init(char **p_where TSRMLS_DC)
 	}
 	memset(stuff, 0, sizeof(*stuff));
 
-	where = "CoCreateInstance";
-	hr = CoCreateInstance(&CLSID_CorRuntimeHost, NULL, CLSCTX_ALL,
-			&IID_ICorRuntimeHost, (LPVOID*)&stuff->dotnet_host);
-
+	where = "dotnet_bind_runtime";
+	hr = dotnet_bind_runtime((LPVOID*)&stuff->dotnet_host);
 	if (FAILED(hr))
 		goto out;
 
@@ -146,7 +178,7 @@ static HRESULT dotnet_init(char **p_where TSRMLS_DC)
 	hr = ICorRuntimeHost_Start(stuff->dotnet_host);
 	if (FAILED(hr))
 		goto out;
-	
+
 	where = "ICorRuntimeHost_GetDefaultDomain";
 	hr = ICorRuntimeHost_GetDefaultDomain(stuff->dotnet_host, &unk);
 	if (FAILED(hr))
@@ -156,7 +188,7 @@ static HRESULT dotnet_init(char **p_where TSRMLS_DC)
 	hr = IUnknown_QueryInterface(unk, &IID_mscorlib_System_AppDomain, (LPVOID*)&stuff->dotnet_domain);
 	if (FAILED(hr))
 		goto out;
-		
+
 	COMG(dotnet_runtime_stuff) = stuff;
 
 out:
@@ -183,12 +215,12 @@ out:
 }
 
 /* {{{ com_dotnet_create_instance - ctor for DOTNET class */
-PHP_FUNCTION(com_dotnet_create_instance)
+PHP_METHOD(dotnet, __construct)
 {
 	zval *object = getThis();
 	php_com_dotnet_object *obj;
 	char *assembly_name, *datatype_name;
-	int assembly_name_len, datatype_name_len;
+	size_t assembly_name_len, datatype_name_len;
 	struct dotnet_runtime_stuff *stuff;
 	OLECHAR *oleassembly, *oletype;
 	BSTR oleassembly_sys, oletype_sys;
@@ -196,37 +228,67 @@ PHP_FUNCTION(com_dotnet_create_instance)
 	int ret = FAILURE;
 	char *where = "";
 	IUnknown *unk = NULL;
+	zend_long cp = GetACP();
+	const struct php_win32_cp *cp_it;
 
-	php_com_initialize(TSRMLS_C);
-	if (COMG(dotnet_runtime_stuff) == NULL) {
-		hr = dotnet_init(&where TSRMLS_CC);
+	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS(), "ss|l",
+			&assembly_name, &assembly_name_len,
+			&datatype_name, &datatype_name_len,
+			&cp)) {
+		RETURN_THROWS();
+	}
+
+	php_com_initialize();
+	stuff = (struct dotnet_runtime_stuff*)COMG(dotnet_runtime_stuff);
+	if (stuff == NULL) {
+		hr = dotnet_init(&where);
 		if (FAILED(hr)) {
 			char buf[1024];
 			char *err = php_win32_error_to_msg(hr);
 			snprintf(buf, sizeof(buf), "Failed to init .Net runtime [%s] %s", where, err);
-			if (err)
-				LocalFree(err);
-			php_com_throw_exception(hr, buf TSRMLS_CC);
+			php_win32_error_msg_free(err);
+			php_com_throw_exception(hr, buf);
+			RETURN_THROWS();
+		}
+		stuff = (struct dotnet_runtime_stuff*)COMG(dotnet_runtime_stuff);
+
+	} else if (stuff->dotnet_domain == NULL) {
+		where = "ICorRuntimeHost_GetDefaultDomain";
+		hr = ICorRuntimeHost_GetDefaultDomain(stuff->dotnet_host, &unk);
+		if (FAILED(hr)) {
+			char buf[1024];
+			char *err = php_win32_error_to_msg(hr);
+			snprintf(buf, sizeof(buf), "Failed to re-init .Net domain [%s] %s", where, err);
+			php_win32_error_msg_free(err);
+			php_com_throw_exception(hr, buf);
 			ZVAL_NULL(object);
-			return;
+			RETURN_THROWS();
+		}
+
+		where = "QI: System._AppDomain";
+		hr = IUnknown_QueryInterface(unk, &IID_mscorlib_System_AppDomain, (LPVOID*)&stuff->dotnet_domain);
+		if (FAILED(hr)) {
+			char buf[1024];
+			char *err = php_win32_error_to_msg(hr);
+			snprintf(buf, sizeof(buf), "Failed to re-init .Net domain [%s] %s", where, err);
+			php_win32_error_msg_free(err);
+			php_com_throw_exception(hr, buf);
+			ZVAL_NULL(object);
+			RETURN_THROWS();
 		}
 	}
 
-	stuff = (struct dotnet_runtime_stuff*)COMG(dotnet_runtime_stuff);
-
 	obj = CDNO_FETCH(object);
 
-	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ss|l",
-			&assembly_name, &assembly_name_len,
-			&datatype_name, &datatype_name_len,
-			&obj->code_page)) {
-		php_com_throw_exception(E_INVALIDARG, "Could not create .Net object - invalid arguments!" TSRMLS_CC);
-		ZVAL_NULL(object);
-		return;
+	cp_it = php_win32_cp_get_by_id((DWORD)cp);
+	if (!cp_it) {
+		php_com_throw_exception(E_INVALIDARG, "Could not create .Net object - invalid codepage!");
+		RETURN_THROWS();
 	}
+	obj->code_page = (int)cp_it->id;
 
-	oletype = php_com_string_to_olestring(datatype_name, datatype_name_len, obj->code_page TSRMLS_CC);
-	oleassembly = php_com_string_to_olestring(assembly_name, assembly_name_len, obj->code_page TSRMLS_CC);
+	oletype = php_com_string_to_olestring(datatype_name, datatype_name_len, obj->code_page);
+	oleassembly = php_com_string_to_olestring(assembly_name, assembly_name_len, obj->code_page);
 	oletype_sys = SysAllocString(oletype);
 	oleassembly_sys = SysAllocString(oleassembly);
 	where = "CreateInstance";
@@ -283,20 +345,17 @@ PHP_FUNCTION(com_dotnet_create_instance)
 		char buf[1024];
 		char *err = php_win32_error_to_msg(hr);
 		snprintf(buf, sizeof(buf), "Failed to instantiate .Net object [%s] [0x%08x] %s", where, hr, err);
-		if (err && err[0]) {
-			LocalFree(err);
-		}
-		php_com_throw_exception(hr, buf TSRMLS_CC);
-		ZVAL_NULL(object);
-		return;
+		php_win32_error_msg_free(err);
+		php_com_throw_exception(hr, buf);
+		RETURN_THROWS();
 	}
 }
 /* }}} */
 
-void php_com_dotnet_mshutdown(TSRMLS_D)
+void php_com_dotnet_mshutdown(void)
 {
 	struct dotnet_runtime_stuff *stuff = COMG(dotnet_runtime_stuff);
-	
+
 	if (stuff->dotnet_domain) {
 		IDispatch_Release(stuff->dotnet_domain);
 	}
@@ -309,10 +368,10 @@ void php_com_dotnet_mshutdown(TSRMLS_D)
 	COMG(dotnet_runtime_stuff) = NULL;
 }
 
-void php_com_dotnet_rshutdown(TSRMLS_D)
+void php_com_dotnet_rshutdown(void)
 {
 	struct dotnet_runtime_stuff *stuff = COMG(dotnet_runtime_stuff);
-	
+
 	if (stuff->dotnet_domain) {
 		IDispatch_Release(stuff->dotnet_domain);
 		stuff->dotnet_domain = NULL;
